﻿using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using NSubstitute;
using Robware.Api.Auth.Controllers;
using Robware.Auth.API;
using System;
using Xunit;

namespace Robware.Api.Auth.Tests.Controllers {
	public class ApiControllerTests {
		[Fact]
		public async Task Validate_WithValidKey_ReturnsOk() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(true);
			var apiKeyRepository = Substitute.For<IApiKeys>();
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Validate("key")).Should().BeOfType<OkResult>();
		}

		[Fact]
		public async Task Validate_WithInvalidKey_ReturnsUnauthorised() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			apiKeyValidator.Validate("key").Returns(false);
			var apiKeyRepository = Substitute.For<IApiKeys>();
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Validate("key")).Should().BeOfType<UnauthorizedResult>();
		}

		[Fact]
		public async Task Create_WithName_ReturnsApiKeyJson() {
			var logger = Substitute.For<ILogger<ApiController>>();
			var apiKeyValidator = Substitute.For<IApiKeyValidator>();
			var apiKeyRepository = Substitute.For<IApiKeys>();

			var expectedKey = new ApiKey { 
				Name = "test",
				Key = "test",
				IssueTimestamp = DateTime.Now,
				Enabled = true
			};

			apiKeyRepository.Create("test").Returns(expectedKey);
				
			var controller = new ApiController(logger, apiKeyValidator, apiKeyRepository);
			(await controller.Create("test")).Value.Should().BeEquivalentTo(expectedKey);
		}
	}
}